////////////////////////////////////////////////////////////////////////////////
// Motor Driver for VBC events
// All connected motors get powered at various power levels and direction
// Default speed values can be changed using brain keys.
// Up/Down select the port to modify.
// Pressing Select key allow to change speed using Up/Down
// Select key again return to port select mode
// Pressing Up and Down key for 0.5s results in an emergency stop of all motors
// while keeping the programmed speed values. Press Up or Down to restart motors
// Philippe "Philo" Hurbain - December 2015
////////////////////////////////////////////////////////////////////////////////


void displayValue(int index, int pwr)
{
	int x, y, ychar, ybar;
	char c='>';
	x = (index%6)*21+1;
	y = (index<6) ? 41 : 10;
	ychar = (index<6) ? 32 : 18;
	ybar = (index<6) ? 25 : 2;
	eraseRect(x+1, ybar+20, x+19, ybar);
	if(pwr==0) c = '-';
	else if(pwr<0)
	{
		c = '<';
		pwr = -pwr;
	}
	if(pwr>99) pwr=99;
	displayStringAt(x, y, " %2d", pwr);
	displayStringAt(x+3, ychar, " %c", c);
	fillRect(x+2, ybar+pwr/6, x+4, ybar);
}

void drawCursor(int index, bool type, bool erase)
// type = true : Draw single line box
// type = false : Draw double line box
{
	int x, y;
	x = (index%6)*21;
	y = (index<6) ? 23 : 1;
	if(!erase)
	{
		drawRect(x, y+21, x+20, y);
		if(!type) drawRect(x+1, y+20, x+19, y+1);
	}
	else
	{
		eraseLine (x, y+21, x+20, y+21);
		eraseLine (x, y, x+20, y);
		eraseLine (x, y+21, x, y);
		eraseLine (x+20, y+21, x+20, y);
		if(!type)
		{
			eraseLine (x+1, y+20, x+19, y+20);
			eraseLine (x+1, y+1, x+19, y+1);
			eraseLine (x+1, y+20, x+1, y+1);
			eraseLine (x+19, y+20, x+19, y+1);
		}
	}
}

void SetConnectedMotorSpeed(short nPortIndex, int motorSpeed)
{
	TVexIQDeviceTypes nDeviceType;
	TDeviceStatus nDeviceStatus;
	short nDeviceVersion;

	getVexIqDeviceInfo(nPortIndex, nDeviceType, nDeviceStatus, nDeviceVersion);
	if (nDeviceType == vexIQ_SensorMOTOR)
	{
		setMotorSpeed(nPortIndex,motorSpeed);
	}
}

task main()
{
	int powerLevel[]={40, 40, 70, 70, 100, 100, -40, -40, -70, -70, -100, -100};
	int index=0;
	for(int i=0; i<12; i++)
	{
		displayValue(i, powerLevel[i]);
		SetConnectedMotorSpeed(i, powerLevel[i]);
	}
	drawCursor(0, true, false);
	while(true)
	{
		int emergency=0;
		while(nLCDButtons!=0)
		{
			sleep(10);
			if(nLCDButtons==(kButtonUp+kButtonDown)) emergency++;
			if(emergency>50)
			{
				eraseDisplay();
				for(int i=0; i<12; i++)
				{
					displayValue(i, 0);
					SetConnectedMotorSpeed(i, 0);
				}
				while(nLCDButtons!=0) sleep(10);
				while((nLCDButtons!=kButtonUp)&&(nLCDButtons!=kButtonDown)) sleep(10);
				for(int i=0; i<12; i++)
				{
					displayValue(i, powerLevel[i]);
					SetConnectedMotorSpeed(i, powerLevel[i]);
				}
				while(nLCDButtons!=0) sleep(10);
			}
		}
		clearTimer(T1);
		while(nLCDButtons==0)
		{
			if(time1[T1]>3000)
			{
				drawCursor(index, true, true);
				clearTimer(T1);
			}
			sleep(10);
		}
		drawCursor(index, true, false);
		switch(nLCDButtons)
		{
		case kButtonUp:
			drawCursor(index, true, true);
			index=(index+1)%12;
			drawCursor(index, true, false);
			break;
		case kButtonDown:
			drawCursor(index, true, true);
			if(index==0) index=11;
			else index--;
			drawCursor(index, true, false);
			break;
		case kButtonSelect:
			bool select=false;
			do
			{
				drawCursor(index, false, false);
				emergency=0;
				while(nLCDButtons!=0)
				{
					sleep(10);
					if(nLCDButtons==(kButtonUp+kButtonDown)) emergency++;
					if(emergency>50)
					{
						eraseDisplay();
						for(int i=0; i<12; i++)
						{
							displayValue(i, 0);
							SetConnectedMotorSpeed(i, 0);
						}
						while(nLCDButtons!=0) sleep(10);
						while((nLCDButtons!=kButtonUp)&&(nLCDButtons!=kButtonDown)) sleep(10);
						for(int i=0; i<12; i++)
						{
							displayValue(i, powerLevel[i]);
							SetConnectedMotorSpeed(i, powerLevel[i]);
						}
						while(nLCDButtons!=0) sleep(10);
					}
				}
				while(nLCDButtons==0) sleep(10);
				switch(nLCDButtons)
				{
				case kButtonUp:
					powerLevel[index]+=10;
					if(powerLevel[index]>100) powerLevel[index]=100;
					SetConnectedMotorSpeed(index, powerLevel[index]);
					displayValue(index, powerLevel[index]);
					break;
				case kButtonDown:
					powerLevel[index]-=10;
					if(powerLevel[index]<-100) powerLevel[index]=-100;
					SetConnectedMotorSpeed(index, powerLevel[index]);
					displayValue(index, powerLevel[index]);
					break;
				case kButtonSelect:
					drawCursor(index, false, true);
					drawCursor(index, true, false);
					select=true;
					break;
				}
			} while(!select);
			break;
		default:
			break;
		}
	}
}
